/*
 * EVE_Test.c
 *
 * Created: 13.12.2018 19:59:13
 * Author : Ich
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "tft.h"



volatile uint8_t system_tick = 0;

/* executed every 5 ms */
ISR (TIMER1_COMPA_vect)
{
	system_tick = 42;
}


void init_timer1(void)
{
	/* Timer1 - 5ms @ 11.0592 Mhz cpu-clock */
	TCCR1B = 0x00;	/* stop */
	TCCR1A = 0x00;
	TCCR1C = 0x00;
	OCR1A = 6912;	/* TOP - timer counts to TOP and generates IRQ */
	TIMSK |= (1<<OCIE1A);
	TCCR1B = (1<<WGM12) | (1<<CS11);	/* mode = CTC, prescaler = 8 -> 1,3824 MHz */
}


void init_timer3(void)
{
	/* Timer3 - just running for profiling */
	TCCR3B = 0x00;	/* stop */
	TCCR3A = 0x00; /* mode: normal */
	TCCR1C = 0x00;
}


void init_ports(void)
{
	DDRA = 0x00;	/* all pins set to input */
	PORTA = 0xff;	/* pullup-resistors activated */

	DDRB = (1<<PB0) | (1<<PB1) | (1<<PB2) | (1<<PB4);	/* SS (not used), SCK, MOSI, PB4=CS, set to Output, others to set to input */
	PORTB = (1<<PB0) | (1<PB5) | (1<<PB3) | (1<<PB6) | (1<<PB7);	/* SS (not used) set to high, CS set to high, pullup-resistors for unused pins activated */

	DDRC = 0x00;
	PORTC = 0xff;

	DDRD = 0x00;
	PORTD = 0x9f;

	DDRE = (1<<PE6); /* PE6 = PowerDown */
	PORTE = 0xff;

	DDRF = 0x00;
	PORTF = 0xff;

	DDRG = 0x00;
	PORTG = 0xff;
}


void init_spi(void)
{
	SPCR = (1<<SPE) | (1<<MSTR) | (1<<SPR1) | (1<<SPR0);	/* SPI on, MSB first, Mode 0, Master, Fosc/4 */
	SPSR = (1<<SPI2X);	/* Fosc/2 */
}


int main(void)
{
	uint8_t display_delay = 0;
	uint16_t counter_val;

	cli();
	init_ports();
	init_timer1();
	init_timer3();
	init_spi();
	sei();

	TFT_init();

	set_sleep_mode(SLEEP_MODE_IDLE);

	while(1)
	{

		if(system_tick)
		{
			system_tick = 0;

			TCNT3 = 0;
			TCCR3B = (1<<CS31); /* start timer, prescaler = 8 -> 723.37ns per tick - autsch */
			TFT_touch();
			TCCR3B = 0; /* stop timer */
			
			counter_val = TCNT3;
			counter_val /= 2; /* some wrong value but not the time in s...  */
			num_profile_b = counter_val;

			display_delay++;
			if(display_delay > 3)
			{
				display_delay = 0;

				TCNT3 = 0;
				TCCR3B = (1<<CS31); /* start timer, prescaler = 8 -> 723.37ns per tick - autsch */
				TFT_display();
				TCCR3B = 0; /* stop timer */

				counter_val = TCNT3;
				counter_val /= 2; /*some wrong value but not the time in s... */
				num_profile_a = counter_val;
			}
		}

		cli();
		sleep_enable();
		sei();
		sleep_cpu();
		sleep_disable();
		} /* while */
	}
